function OUT=NoiseVar_CRC;

% Estimating noise variance for image with additive uncorrelated noise
% Corr(0) is extrapolated from Cor(1)and Cor(2)through parabolic approximation 
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Usage: OUT=NoiseVar_CRC

% INPIMG=load_img_CRC;

m=menu('Select test image','Interferogram','Space Photograph');
switch m,
    case 1, load interfero; INPIMG=interfero;
    case 2, load space_photo; INPIMG=space_photo;
end

INPIMG=INPIMG-mean(INPIMG(:));
[SzX SzY]=size(INPIMG);
n=menu('Add noise with standard deviation','0','10','20','30','40','50');
noise=10*(n-1)*randn(SzX,SzY);
[mn,NoiseStd]=std2d(noise);
INPIMG=INPIMG+noise;

figure__n(2);
figure(1);
display_1(INPIMG);
title(['Input image. Noise stand. dev. = ',num2str(NoiseStd)],'FontWeight','Bold')
drawnow
cor=fftshift(corimg_1d(INPIMG));
OUT=sqrt(cor(1)-4*cor(2)/3+cor(3)/3);% Parabolic extrapolation

figure(2);
bar(cor(1:fix(SzX/16)));
axis tight
grid on
title(['1-D correlation function. Etimated noise stand. dev. =',num2str(OUT)],'FontWeight','Bold');
xlabel('Displacement','FontWeight','Bold')
set(gca,'FontWeight','Bold')
drawnow




%------------------------------------------------------------
function display_1(INPIMG)
% Displays a 2-D signal as an image 
% with normalization by gobal maximum and minimum
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call display1(INPIMG)

clf
colormap(gray(256))
Min=min(min(INPIMG)); Max=max(max(INPIMG));
if Max-Min==0, im=zeros(size(INPIMG)); else im=255*(INPIMG-Min)/(Max-Min); end

image(im);axis image;axis off;

%------------------------------------------------------------
function corr=corimg_1d(INPIMG);

% 1D autocorrelation function of an image: 
% inverse FFT of averaged spectra of image columns
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call corr=corimg1d(INPIMG);

[SzX,SzY]=size(INPIMG);
INPIMG=INPIMG-mean(mean(INPIMG))*ones(size(INPIMG));
spimg=(abs(fft(INPIMG))).^2;

meansp=mean(spimg');
corr=fftshift(abs(ifft(meansp)))/SzX;

%-----------------------------------------------
function figure__n(N)
% Generating N square figure windows of ScreenVerSz/2*ScreenVerSz pixels
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Version July 15, 2001
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call figure_n(N)

% Generating a figure window
ScrSz=get(0,'ScreenSize');
HorSz=(ScrSz(3)-20); VerSz=(ScrSz(4));
close all;

switch N;
case 1, 
   Nvert=1;Nhor=1;H_Sz=(HorSz-80); V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-V_Sz V_Sz V_Sz]); colormap(gray(256))
case 2, 
   Nvert=1;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
case 3, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 4, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 5,
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 6, 
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-75-2*V_Sz H_Sz V_Sz]);colormap(gray(256))
end
%--------------------------------------------------------
function OUTIMG=load_img_CRC;

% A browser for loading images in all formats
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUTIMG=load_img_CRC;

% dr=pwd;
% cd c:\yaro_data\images\advlabdemo;
[FILENAME, PATHNAME] = uigetfile('*.*', 'LOAD IMAGE in *.tif format');

% eval(['cd ',dr]);

OUTIMG=imread([PATHNAME FILENAME]);
OUTIMG=double(OUTIMG);