function [pw,pw_n,pw_f,f_mask]=demo_lcdct2_CRC;

% Local adaptive DCT domain filtering demo
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Dec. 22, 2012


figure_n(4);
load pw;
nl=menu('Select noise level NoiseStDev','5','10','15','20','25','30','40','45','50');
NoiseStDev=5*nl;
pw_n=pw+NoiseStDev*randn(size(pw));

t=1;
while t==1,
    ThrFact=menu('Select filtering threshold factor (Thr=ThrFact*SignaNoise)','0.5','0.75','1','1.25','1.5');
    Thr=(0.25*ThrFact+0.25)*NoiseStDev;%15
    [mn, NoiseStDev]=std2d(pw-pw_n);
    
    figure(1);
    imshow(pw_n,[]);
    title(['Noisy test image; NoiseStDev=',num2str(NoiseStDev)],'FontWeight','Bold');
    drawnow

    WSz=menu('Select window size','5x5','7x7','9x9','11x11','13x13','15x15','17x17','19x19','21x21');
    WSz=2*WSz+3;

    [pw_f,f_mask]=lcdct_2(pw_n,ones(WSz),1,1,Thr);
    t=menu('Repeat filtering with other parameters?','Yes','No');
end

%-------------------------------------------------------
function [OUTIMG1,OUTIMG2]=lcdct_2(INPIMG,sp_mask,p,g,thr);

% LOCAL DCT FILTERING OVER WINDOW (SzWx,SzWy)=size(sp_mask);
% Version Aug. 1, 1999
% p - power to which magnitudes of spectral coefficients are rised
% g -amplification coefficient for modified spectral coefficients  
% thr - threshold with which magnitudes of spectral coefficients are compared
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUTIMG=lcdct2(INPIMG,sp_mask,p,g,thr);

[SzWx SzWy]=size(sp_mask);
W=SzWx*SzWy;
THR=thr*sqrt(SzWx);
[SzX SzY]=size(INPIMG);
k=menu('Choose the filter','Rejective filter','Empirical Wiener filter');
OUTIMG1=INPIMG;
OUTIMG2=INPIMG;
% figure(1)
% display1(OUTIMG1)
% title('Input image');
% drawnow

imgext=img_ext(INPIMG,fix((SzWx+1)/2),fix((SzWy+1)/2));

r=0:SzWx-1;s=0:SzWy-1;
recon_mask_x=(cos(pi*r*(SzWx+1)/(2*SzWx)));
recon_mask_y=(cos(pi*s'*(SzWy+1)/(2*SzWy)));
recon_msk=kron(recon_mask_x,recon_mask_y);
recon_msk(1,:)=recon_msk(1,:)/2;
recon_msk(:,1)=recon_msk(:,1)/2;

for X=1:SzX
   uicontrol(gcf,'Style',...
   'text','Position',[1 1 150 25],...
   'String',[num2str(X),'-th row; ']);
   drawnow
   for Y=1:SzY,
        fragm=imgext(X:SzWx+X-1,Y:SzWy+Y-1);
        alpha=dct2(fragm);
%FILTER
        dc=alpha(1,1);
        alpha_sign=sign(alpha); 
        alpha_abs=abs(alpha);
        alpha_abs(1,1)=0;
        norm1=sum(sum(alpha_abs.^2));
        if k==1,
        mask=alpha_abs>=THR;
        else	
        alpha_abs(1,1)=dc;
        alpha_2=alpha_abs.^2;
        alpha_2_1=alpha_2-THR.^2; alpha_2_1=alpha_2.*(alpha_2_1>0);
        mask=alpha_2_1./(alpha_2+eps); 
        end
        R=(sum(sum(mask))/W).^0.5;
        alpha_mod=(alpha_abs.*mask).^p;	
        alpha_abs(1,1)=0;
        norm2=sum(sum((alpha_abs.^(p)).^2))+eps;
        alpha_mod=g*alpha_mod*sqrt(norm1/norm2);
        alpha_mod(1,1)=dc;
        alpha=alpha_sign.*alpha_mod;
        alpha=alpha.*sp_mask;	
        out=idct2(alpha);
        OUTIMG1(X,Y)=out(fix((SzWx+1)/2)+1,fix((SzWy+1)/2)+1);
        OUTIMG2(X,Y)=255*R;
	end

   if X/16-round(X/16)==1/16,
   figure(2);
   imshow(OUTIMG1,[]);
   title(['Filtered image ; SzW=',num2str(SzWx),'x',num2str(SzWy),';p=',num2str(p),';g=',num2str(g),';thr=',num2str(thr)],'FontWeight','Bold');
   drawnow
   figure(3);
   imshow(INPIMG-OUTIMG1,[]);
   title('Filtered image component','FontWeight','Bold');
   drawnow
   figure(4);
   imshow(OUTIMG2,[]);
   title(['Filter transparentness; WSz=',num2str(SzWx),'x',num2str(SzWy)],'FontWeight','Bold');
   drawnow
   end
end

figure(2);
imshow(OUTIMG1,[]);
title(['Filtered image ; SzW=',num2str(SzWx),'x',num2str(SzWy),';p=',num2str(p),';g=',num2str(g),';thr=',num2str(thr)],'FontWeight','Bold');
drawnow
figure(3);
imshow(INPIMG-OUTIMG1,[]);
[mn,StDev]=std2d(INPIMG-OUTIMG1);
title(['Filtered out image component; StDev=',num2str(StDev)],'FontWeight','Bold');
drawnow
figure(4);
imshow(OUTIMG2,[]);%colormap(jet);colorbar('horiz');
title(['Filter transparentness; WSz=',num2str(SzWx),'x',num2str(SzWy)],'FontWeight','Bold');
drawnow

%------------------------------------------------------------------------------------------
function figure_n(N)
% Generating N square figure windows of ScreenVerSz/2*ScreenVerSz pixels
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Version July 15, 2001
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call figure_n(N)

% Generating a figure window
ScrSz=get(0,'ScreenSize');
HorSz=(ScrSz(3)-20); VerSz=(ScrSz(4));
close all;

switch N;
case 1, 
   Nvert=1;Nhor=1;H_Sz=(HorSz-80); V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-V_Sz V_Sz V_Sz]); colormap(gray(256))
case 2, 
   Nvert=1;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
case 3, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 4, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 5,
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 6, 
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-75-2*V_Sz H_Sz V_Sz]);colormap(gray(256))
end
%--------------------------------------------------------------------

function [mn,v]=std2d(INP);

% Standard deviation and mean of a 2-D array
% z=[mean stdeviation];
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call [mn v]=std2d(INP)

[SzX SzY]=size(INP);
mn=sum(sum(INP))/(SzX*SzY);
v=sqrt(sum(sum((INP-mn).^2))/(SzX*SzY));




